﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ConsoleAppLevel2Class
{
    public class GarbageCollectionDemo
    {
        /*
            Garbage Collection Class Memory
            Threads Processor Chapter 12

            3 Step Process to Manage Memory "free up memory"
            1. Mark what is being used currently by the application
            2. Move the needed objects to a new "heap" "compacting"
            3. Collecting removes unneeded object

            3 Generations of Memory
            Generation 0: new objects GC automatically
            Generation 1: objs get moved based on 3 step process "saved" buffer between Gen 0 (short lived) and Gen 2 (long lived)
            Generation 2: long lived objects. "very resource intensive"

           Triggers that kick off the GC Process
           1. Allocated size is exceeded by the heap size
           2. System memory is low
           3. GC.Collect() is explicitly called by the code   


        */ // Notes

        public static void RunDemo()
        {
            // Display initial memory used
            Console.WriteLine("Initial Memory: " + GC.GetTotalMemory(false) + " bytes.");

            // Add to the memory size
            int[] largeArray = new int[1000000]; // increase our memory size
            
            // Display current memory used
            Console.WriteLine("Current Memory: " + GC.GetTotalMemory(false) + " bytes."); // much larger than the initial

            // largeArray is being used to hold int[1000000]

            // Set the largeArray to null to make it eligibe for GC
            largeArray = null; // easy way to get rid of objs

            // Simulate unmanaged resources that we can dispose
            ResourceManager resourceManager = new ResourceManager();
            // ptr 123456
            // resourceManager.ManipulateResourceDirectly();
            // ptr 789
            resourceManager.Dispose();
            // ptr 0


            // Manually force a GC on managed resources
            GC.Collect(); // does not destruct unmanaged resources such as db conn, file i/o's file handlers...

            // Ensure that all the finalizers run on unmanaged resources
            GC.WaitForPendingFinalizers();

            // Display current memory used
            Console.WriteLine("Memory after GC: " + GC.GetTotalMemory(false) + " bytes."); // smaller than 2nd time


        } // RunDemo()

    } // GarbageCollectionDemo

} // ns
